@@ -165,7 +165,7 @@ computer_clock_update_func (ClockData * cd,
 	ComputerClock *cc;
 	struct tm *tm;
 	GString *gs = g_string_new("");
-	char date[256], hour[256], tooltip[256];
+	char date[256], hour[256];
 	char *utf8;
 	
 	cc = g_object_get_data (G_OBJECT (cd->clockw), "cc");
@@ -288,6 +288,7 @@ create_computer_clock_widget (GtkWidget ** clock, ClockUpdateFunc * update_func)
 	gtk_widget_show (vbox);
 
 	cc->time = gtk_label_new ("hmm?");
+	gtk_label_set_justify (GTK_LABEL (cc->time), GTK_JUSTIFY_CENTER);
 
 	gtk_box_pack_start_defaults (GTK_BOX (vbox), cc->time);
 	gtk_widget_show (cc->time);
